@@ -5,25 +5,60 @@ use strict;
 use warnings;
 
 use POE qw(
+    Component::Client::TCP
     Component::Server::TCP
 );
 use Sys::Hostname;
-use AnyEvent::Graphite;
 
-our $VERSION = '1.8';
+our $VERSION = '2.0';
 
 my @_STREAM_NAMES = qw(subscribers match debug full regex);
 my %_STREAM_ASSISTERS = (
     subscribers => 'programs',
-    match => 'words',
+    match       => 'words',
 );
 
 
 # Precompiled Regular Expressions
 my %_PRE = (
-    program => qr/\s+\d+:\d+:\d+\s+\S+\s+([^:\s]+)(:|\s)/,
+    program => qr/\d{1,2}:\d{1,2}:\d{1,2}(?:\.\d{3,})?(?:\+\d{1,2}:\d{1,2})?\s+\S+\s+([^:\s]+)(:|\s)/,
 );
 
+sub _benchmark_regex {
+    eval {
+        require Benchmark;
+    };
+    if( $@ ) {
+        warn "unable to load Benchmark.pm: $@";
+        return;
+    }
+    my @msgs = (
+        q|<11>Jan  1 00:00:00 mainfw snort[32640]: [1:1893:4] SNMP missing community string attempt [Classification: Misc Attack] [Priority: 2]: {UDP} 1.2.3.4:23210 -> 5.6.7.8:161|,
+        q|<11>Jan  1 00:00:00 11.22.33.44 dhcpd: DHCPINFORM from 172.16.2.137 via vlan3|,
+        q|Jan  1 00:00:00 11.22.33.44 dhcpd: DHCPINFORM from 172.16.2.137 via vlan3|,
+        q|<11>Jan  1 00:00:00 dev.example.com dhcpd: DHCPINFORM from 172.16.2.137 via vlan3|,
+        q|Jan  1 00:00:00 example syslogd 1.2.3: restart (remote reception).|,
+        q|<163>Jun 7 18:39:00 hostname.domain.tld %ASA-3-313001: Denied ICMP type=5, code=1 from 1.2.3.4 on interface inside|,
+        q|2016-11-29T06:30:01+01:00 ether CROND[15084]: (root) CMD (/usr/lib64/sa/sa1 1 1)|,
+    );
+    my %tests = ();
+    my %misses = ();
+    foreach my $re (keys %_PRE) {
+       $tests{$re} = sub {
+            for (@msgs) {
+                if( $_ !~ /$_PRE{$re}/ ) {
+                    $misses{"$re | $_"}=1;
+                }
+            }
+        };
+    }
+    Benchmark::cmpthese(500_000, \%tests);
+    if( keys %misses ) {
+        print "\nREGEX MISSES:\n\n";
+        print "  $_\n" for sort keys %misses;
+    }
+}
+
 
 
 sub spawn {
@@ -133,15 +168,37 @@ sub dispatcher_start {
 sub graphite_connect {
     my ($kernel,$heap) = @_[KERNEL,HEAP];
 
-    eval {
-        $heap->{_graphite} = AnyEvent::Graphite->new(
-            host => $heap->{config}{GraphiteHost},
-            port => $heap->{config}{GraphitePort},
-        );
-    };
-    if( my $err = $@ ) {
-        debug("Graphite server setup failed: $err");
-    }
+    # Build the Graphite Handler
+    $heap->{_graphite} = POE::Component::Client::TCP->new(
+        Alias => 'graphite',
+        RemoteHost => $heap->{config}{GraphiteHost},
+        RemotePort => $heap->{config}{GraphitePort},
+        ConnectTimeout => 5,
+        Connected => sub {
+            # let the parent know we're able to write
+            $heap->{graphite} = 1;
+        },
+        ConnectError => sub {
+            my ($op,$err_num,$err_str) = @_[ARG0..ARG2];
+            delete $heap->{graphite} if exists $heap->{graphite};
+            $heap->{stats}{graphite_errors} ||= 0;
+            $heap->{stats}{graphite_errors}++;
+            # Attempt to reconnect
+            $kernel->delay( reconnect => 60 );
+        },
+        Disconnected => sub {
+            $kernel->delay( reconnect => 60  );
+        },
+        Filter => "POE::Filter::Line",
+        InlineStates => {
+            send => sub {
+                $_[HEAP]->{server}->put($_[ARG0]);
+            },
+        },
+        ServerError => sub {
+            $_[KERNEL]->yield( 'reconnect' );
+        },
+    );
 }
 
 #--------------------------------------------------------------------------#
@@ -152,21 +209,14 @@ sub flush_stats {
 
     if (exists $heap->{stats}) {
         my $stats = delete $heap->{stats};
-        if( exists $heap->{_graphite} && defined $heap->{_graphite} ) {
+        if( exists $heap->{graphite} && $heap->{graphite} ) {
             my $time = time();
             foreach my $stat (keys %{ $stats }) {
                 my $metric = join('.', $heap->{config}{GraphitePrefix}, $heap->{hostname}, $stat);
-                eval {
-                    $heap->{_graphite}->send("$metric", $stats->{$stat}, $time);
-                };
-                if( my $err = $@ ) {
-                    debug("Error sending statistics, reconnecting.");
-                    $kernel->yield('graphite_connect');
-                    last;
-                }
+                $kernel->post( graphite => send => join " ", $metric, $stats->{$stat}, $time);
             }
         }
-        debug("STATS: " . join(", ", map { "$_:$stats->{$_}" } keys %{ $stats } ) ); #"
+        debug('STATS: ' . join(', ', map { "$_:$stats->{$_}" } keys %{ $stats } ) );
     }
     $heap->{stats} = {
         map { $_ => 0 } qw(received received_bytes dispatched dispatched_bytes)
@@ -183,6 +233,7 @@ sub dispatch_message {
     _dispatch_messages($kernel, $heap, [$msg]);
 }
 
+
 sub dispatch_messages {
     my ($kernel,$heap,$msgs) = @_[KERNEL,HEAP,ARG0];
 
@@ -208,7 +259,7 @@ sub _dispatch_messages {
         $heap->{stats}{received_bytes} += length $msg;
 
         # Program based subscriptions
-        if( my ($program) = map { lc } ($msg =~ /$_PRE{program}/) ) {
+        if( my ($program) = map { lc } ($msg =~ /$_PRE{program}/o) ) {
             # remove the sub process and PID from the program
             $program =~ s/\(.*//g;
             $program =~ s/\[.*//g;
@@ -286,7 +337,7 @@ sub register_client {
 sub debug_client {
     my ($kernel,$heap,$sid) = @_[KERNEL,HEAP,ARG0];
 
-    remove_stream($heap,$sid,'full');
+    _remove_stream($heap,$sid,'full');
 
     $heap->{debug}{$sid} = 1;
     $kernel->post( $sid => 'client_print' => 'Debugging enabled.' );
@@ -297,7 +348,7 @@ sub debug_client {
 sub nobug_client {
     my ($kernel,$heap,$sid) = @_[KERNEL,HEAP,ARG0];
 
-    remove_stream($heap,$sid,'debug');
+    _remove_stream($heap,$sid,'debug');
 
     $kernel->post( $sid => 'client_print' => 'Debugging disabled.' );
 }
@@ -307,7 +358,7 @@ sub nobug_client {
 sub fullfeed_client {
     my ($kernel,$heap,$sid) = @_[KERNEL,HEAP,ARG0];
 
-    remove_all_streams($heap,$sid);
+    _remove_all_streams($heap,$sid);
 
     # Add to fullfeed:
     $heap->{full}{$sid} = 1;
@@ -316,10 +367,11 @@ sub fullfeed_client {
 }
 #--------------------------------------------------------------------------#
 
+
 sub nofullfeed_client {
     my ($kernel,$heap,$sid) = @_[KERNEL,HEAP,ARG0];
 
-    remove_all_streams($heap,$sid);
+    _remove_all_streams($heap,$sid);
 
     $kernel->post( $sid => 'client_print' => 'Full feed disabled.');
 }
@@ -329,7 +381,7 @@ sub nofullfeed_client {
 sub subscribe_client {
     my ($kernel,$heap,$sid,$argstr) = @_[KERNEL,HEAP,ARG0,ARG1];
 
-    remove_stream($heap,$sid,'full');
+    _remove_stream($heap,$sid,'full');
 
     my @progs = map { lc } split /[\s,]+/, $argstr;
     foreach my $prog (@progs) {
@@ -360,7 +412,7 @@ sub unsubscribe_client {
 sub match_client {
     my ($kernel,$heap,$sid,$argstr) = @_[KERNEL,HEAP,ARG0,ARG1];
 
-    remove_stream($heap,$sid,'full');
+    _remove_stream($heap,$sid,'full');
 
     my @words = map { lc } split /[\s,]+/, $argstr;
     foreach my $word (@words) {
@@ -372,6 +424,7 @@ sub match_client {
 }
 #--------------------------------------------------------------------------#
 
+
 sub flush_client {
     my ($kernel, $heap) = @_[KERNEL, HEAP];
 
@@ -435,7 +488,7 @@ sub regex_client {
 sub noregex_client {
     my ($kernel,$heap,$sid) = @_[KERNEL,HEAP,ARG0];
 
-    remove_stream($heap,$sid,'regex');
+    _remove_stream($heap,$sid,'regex');
 
     $kernel->post( $sid => 'client_print' => 'No longer receving regex-based matches' );
 }
@@ -528,14 +581,14 @@ sub hangup_client {
     delete $heap->{clients}{$sid};
     delete $heap->{buffers}{$sid};
 
-    remove_all_streams($heap,$sid);
+    _remove_all_streams($heap,$sid);
 
     debug("Client Termination Posted: $sid\n");
 
 }
 
 #--------------------------------------------------------------------------#
-sub remove_stream {
+sub _remove_stream {
     my ($heap,$sid,$stream) = @_;
 
     debug("Removing '$stream' for $sid");
@@ -553,11 +606,11 @@ sub remove_stream {
     }
 }
 
-sub remove_all_streams {
+sub _remove_all_streams {
     my ($heap,$sid) = @_;
 
     foreach my $stream (@_STREAM_NAMES) {
-        remove_stream($heap,$sid,$stream);
+        _remove_stream($heap,$sid,$stream);
     }
 }
 #--------------------------------------------------------------------------#
@@ -738,8 +791,6 @@ sub client_term {
 }
 
 
-#--------------------------------------------------------------------------#
-
 
 1;
 
@@ -747,13 +798,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 POE::Component::Server::eris - POE eris message dispatcher
 
 =head1 VERSION
 
-version 1.8
+version 2.0
 
 =head1 SYNOPSIS
 
@@ -792,11 +845,7 @@ rsyslog are included in the examples directory!
 
     POE::Kernel->run();
 
-=head1 EXPORT
-
-POE::Component::Server::eris does not export any symbols.
-
-=head1 FUNCTIONS
+=head1 METHODS
 
 =head2 spawn
 
@@ -806,156 +855,6 @@ Parameters:
     ListenAddress           => 'localhost',         #default
     ListenPort              => '9514',              #default
 
-=head2 INTERNAL Subroutines (Events)
-
-=head3 debug
-
-Controls Debugging Output to the controlling terminal
-
-=head3 dispatcher_start
-
-Sets the alias and creates in-memory storages
-
-=head3 graphite_connect
-
-Establish a connection to the graphite server
-
-=head3 flush_stats
-
-Send statistics to the graphite server and the debug clients
-
-=head3 dispatch_message
-
-Based on clients connected and their feed settings, distribute this message
-
-=head3 server_error
-
-Handles errors related to the PoCo::TCP::Server
-
-=head3 register_client
-
-Client Registration for the dispatcher
-
-=head3 debug_client
-
-Enables debugging for the client requesting it
-
-=head3 nobug_client
-
-Disables debugging for a particular client
-
-=head3 fullfeed_client
-
-Adds requesting client to the list of full feed clients
-
-=head3 subscribe_client
-
-Handle program name subscription
-
-=head3 unsubscribe_client
-
-Handle unsubscribe requests from clients
-
-=head3 match_client
-
-Handle requests for string matching from clients
-
-=head3 nomatch_client
-
-Remove a match based feed from a client
-
-=head3 regex_client
-
-Handle requests for string regexes from clients
-
-=head3 noregex_client
-
-Remove a match based feed from a client
-
-=head3 status_client
-
-Send current server statistics to client
-
-=head3 dump_client
-
-Dump something interesting to the client
-
-=head3 hangup_client
-
-This handles cleaning up from a client disconnect
-
-=head3 server_shutdown
-
-Announce server shutdown, shut off PoCo::Server::TCP Session
-
-=head3 client_connect
-
-PoCo::Server::TCP Client Establishment Code
-
-=head3 client_print
-
-PoCo::Server::TCP Write to Client
-
-=head3 broadcast
-
-PoCo::Server::TCP Broadcast Messages
-
-=head3 debug_message
-
-Send debug message to DEBUG clients
-
-=head3 client_input
-
-Parse the Client Input for eris::dispatcher commands and enact those commands
-
-=head3 client_term
-
-PoCo::Server::TCP Client Termination
-
-=head1 AUTHOR
-
-Brad Lhotsky, C<< <brad.lhotsky at gmail.com> >>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-poe-component-server-eris at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=POE-Component-Server-eris>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc POE::Component::Server::eris
-
-You can also look for information at:
-
-=over 4
-
-=item * Github
-
-L<https://github.com/reyjrar/POE-Component-Server-eris>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/POE-Component-Server-eris>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/POE-Component-Server-eris>
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=POE-Component-Server-eris>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/POE-Component-Server-eris>
-
-=back
-
 =head1 ACKNOWLEDGEMENTS
 
 =over 4
@@ -966,14 +865,166 @@ L<http://search.cpan.org/dist/POE-Component-Server-eris>
 
 =head1 AUTHOR
 
-Brad Lhotsky <brad.lhotsky@gmail.com>
+Brad Lhotsky <brad@divsionbyzero.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2011 by Brad Lhotsky.
+This software is Copyright (c) 2016 by Brad Lhotsky.
 
 This is free software, licensed under:
 
   The (three-clause) BSD License
 
+=head1 EVENTS
+
+=head2 debug
+
+Controls Debugging Output to the controlling terminal
+
+=head2 dispatcher_start
+
+Sets the alias and creates in-memory storages
+
+=head2 graphite_connect
+
+Establish a connection to the graphite server
+
+=head2 flush_stats
+
+Send statistics to the graphite server and the debug clients
+
+=head2 dispatch_message
+
+Based on clients connected and their feed settings, distribute this message
+
+=head2 dispatch_messages
+
+Splits multiple messages by line feeds into many messages.
+
+=head2 server_error
+
+Handles errors related to the PoCo::TCP::Server
+
+=head2 register_client
+
+Client Registration for the dispatcher
+
+=head2 debug_client
+
+Enables debugging for the client requesting it
+
+=head2 nobug_client
+
+Disables debugging for a particular client
+
+=head2 fullfeed_client
+
+Adds requesting client to the list of full feed clients
+
+=head2 nofullfeed_client
+
+Disables the full feed from the requesting client.
+
+=head2 subscribe_client
+
+Handle program name subscription
+
+=head2 unsubscribe_client
+
+Handle unsubscribe requests from clients
+
+=head2 match_client
+
+Handle requests for string matching from clients
+
+=head2 flush_client
+
+Flushes the outstanding buffer to the client.
+
+=head2 nomatch_client
+
+Remove a match based feed from a client
+
+=head2 regex_client
+
+Handle requests for string regexes from clients
+
+=head2 noregex_client
+
+Remove a match based feed from a client
+
+=head2 status_client
+
+Send current server statistics to client
+
+=head2 dump_client
+
+Dump something interesting to the client
+
+=head2 hangup_client
+
+This handles cleaning up from a client disconnect
+
+=head2 server_shutdown
+
+Announce server shutdown, shut off PoCo::Server::TCP Session
+
+=head2 client_connect
+
+PoCo::Server::TCP Client Establishment Code
+
+=head2 client_print
+
+PoCo::Server::TCP Write to Client
+
+=head2 broadcast
+
+PoCo::Server::TCP Broadcast Messages
+
+=head2 debug_message
+
+Send debug message to DEBUG clients
+
+=head2 client_input
+
+Parse the Client Input for eris::dispatcher commands and enact those commands
+
+=head2 client_term
+
+PoCo::Server::TCP Client Termination
+
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+MetaCPAN
+
+A modern, open-source CPAN search engine, useful to view POD in HTML format.
+
+L<http://metacpan.org/release/POE-Component-Server-eris>
+
+=item *
+
+RT: CPAN's Bug Tracker
+
+The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
+
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=POE-Component-Server-eris>
+
+=back
+
+=head2 Source Code
+
+This module's source code is available by visiting:
+L<https://github.com/reyjrar/POE-Component-Server-eris>
+
 =cut
